Convert a plain matrix or simple triplet form matrix to a Matrix::dsCMatrix (implicit, upper) form
Source:R/sparse.R
make_csc_symm_matrix.RdConvert a plain matrix or simple triplet form matrix to a Matrix::dsCMatrix (implicit, upper) form
Value
a list of row pointer, column pointer, and values corresponding to a Matrix::dsCMatrix object