An S7 class wrapping the PIQP C++ Solver. Users will never
need to directly create instances of this class and should use the
more user-friendly functions piqp() and solve_piqp().
Value
An S7 object of class piqp_model with properties solver_ptr
(an external pointer to the C++ solver), dims (a named list with
elements n, p, and m giving the number of variables, equality
constraints, and inequality constraints), and dense_backend (a logical
flag indicating whether the dense solver backend is in use). The object
is used with base::solve() to compute a solution and with
stats::update() to modify problem data in place.