An S7 class wrapping the PIQP C++ Solver. Users will never
need to directly create instances of this class and should use the
more user-friendly functions piqp() and solve_piqp().
An S7 class wrapping the PIQP C++ Solver. Users will never
need to directly create instances of this class and should use the
more user-friendly functions piqp() and solve_piqp().